package ru.ulstu.fc.user.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.fc.user.model.User;
import ru.ulstu.fc.user.model.UserNotFoundException;
import ru.ulstu.fc.user.model.UserRole;
import ru.ulstu.fc.user.model.UserRoleConstants;
import ru.ulstu.fc.user.repository.UserRepository;
import ru.ulstu.fc.user.repository.UserRoleRepository;
import ru.ulstu.fc.user.utils.UserUtils;

import java.util.Collections;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@Transactional
public class UserService implements UserDetailsService {
    private final Logger log = LoggerFactory.getLogger(UserService.class);
    private final PasswordEncoder passwordEncoder;
    private final UserRepository userRepository;
    private final UserRoleRepository userRoleRepository;
    @Value("${admin-password}")
    private String adminPassword;

    public UserService(PasswordEncoder passwordEncoder,
                       UserRepository userRepository,
                       UserRoleRepository userRoleRepository) {
        this.passwordEncoder = passwordEncoder;
        this.userRepository = userRepository;
        this.userRoleRepository = userRoleRepository;
    }

    public User getUserByLogin(String login) {
        return userRepository.findOneByLoginIgnoreCase(login);
    }

    @Override
    public UserDetails loadUserByUsername(String username) {
        final User user = userRepository.findOneByLoginIgnoreCase(username);
        if (user == null) {
            throw new UserNotFoundException(username);
        }
        return new org.springframework.security.core.userdetails.User(user.getLogin(),
                user.getPassword(),
                Optional.ofNullable(user.getRoles()).orElse(Collections.emptySet()).stream()
                        .map(role -> new SimpleGrantedAuthority(role.getName()))
                        .collect(Collectors.toList()));
    }

    public User createUser(User user) {
        if (getUserByLogin(user.getLogin()) != null) {
            throw new RuntimeException(user.getLogin());
        }
        User dbUser = (user.getId() == null)
                ? user
                : getUserById(user.getId());
        //user.setRoles(Collections.singleton(new UserRole(UserRoleConstants.USER)));
        dbUser.setPassword(passwordEncoder.encode(user.getPassword()));
        dbUser.setLogin(user.getLogin());
        dbUser = userRepository.save(dbUser);
        log.debug("Created Information for User: {}", dbUser.getLogin());
        return dbUser;
    }

    public User getUserById(Integer id) {
        return userRepository.findById(id).orElseThrow(() -> new RuntimeException("User not found by id"));
    }

    private void createDefaultUser(String login, String userRole) {
        if (getUserByLogin(login) == null) {
            UserRole role = userRoleRepository.save(new UserRole(userRole.toString()));
            createUser(new User(login, login.equals("admin") ? adminPassword : login, Set.of(role)));
        }
    }

    public void initDefaultAdmin() {
        createDefaultUser("admin", UserRoleConstants.ADMIN);
    }

    public User getCurrentUser() {
        return getUserByLogin(UserUtils.getCurrentUserLogin());
    }
}
